#!/usr/bin/env python3
import onlinejudge.atcoder
import onlinejudge.yukicoder
import onlinejudge.anarchygolf
import onlinejudge.codeforces
import onlinejudge.hackerrank
import onlinejudge.aoj
import onlinejudge.dispatch
